{Object_Title_and_Purpose}


CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

        Rot1    = 21
        RotCa   = 22
        Rot4    = 23
        Rot8    = 20
        RotCb   = 19
        Rot2    = 18

        WClkOut1 = 0
        WClkOut2 = 1
        WClkOut3 = 2

        DE1     = 3 

        Debug   = 0

        #0, F44_100Hz, F88_200Hz, F132_300Hz, F48_000Hz, F96_000Hz, F144_000Hz, F192_000Hz

        #0, Enable, Disable

        WMin  =       381
        
VAR
  long  ctrReturn, frqReturn

OBJ

  FDS      : "fullDuplexSerial4port.spin"
  DIO      : "dataIO4port.spin"
  CTRCTL[3]   : "CounterControl 1.3.13.spin"
   
PUB Main | returnVal, oldSwitchValue

  'Start terminal debugger.
  waitcnt(clkfreq*2 + cnt)
  FDS.INIT
  fds.AddPort(DEBUG, 31, 30, -1, -1, 0, 0, 57600)
  returnVal := fds.Start
  fds.tx(debug, 16) ' Clear Screen
  fds.str(DEBUG, String("FDS Terminal running on cog "))
  dio.dec(DEBUG, returnVal)
  fds.tx(DEBUG, ".")
  fds.tx(DEBUG, 13)
  
  'waitcnt(clkfreq*2 + cnt)
  
  fds.str(DEBUG, String("Starting counter controller on cog "))
  returnVal := CTRCTL[0].start(WClkOut1, @ctr, @frq, @start, @cntSync)
  dio.dec(DEBUG, returnVal)
  fds.str(DEBUG, String(".", 13))

  fds.str(DEBUG, String("Starting counter controller on cog "))
  returnVal := CTRCTL[1].start(WClkOut2, @ctr, @frq, @start, @cntSync)
  dio.dec(DEBUG, returnVal)
  fds.str(DEBUG, String(".", 13))
  
  fds.str(DEBUG, String("Starting counter controller on cog "))
  returnVal := CTRCTL[2].start(WClkOut3, @ctr, @frq, @start, @cntSync)
  dio.dec(DEBUG, returnVal)
  fds.str(DEBUG, String(".", 13))  '}
  
  long[@start] := 0 

  waitcnt(clkfreq/4 + cnt)
  
  OUTA[RotCa]~  'Set Rotary Switch common pins to ground
  OUTA[RotCb]~
  DIRA[RotCa]~~
  DIRA[RotCb]~~

  DIRA[Rot1]~   'Set Rot Signal pins to inputs
  DIRA[Rot2]~
  DIRA[Rot4]~
  DIRA[Rot8]~
  
  oldSwitchValue := -1

  waitcnt(clkfreq/2 + cnt)
  
  repeat
    returnVal := ReadRotorySwitch
    if oldSwitchValue <> returnVal
    
      oldSwitchValue := returnVal

      DriverControl(Disable)
      
      CASE oldSwitchValue
        F44_100Hz:
          SynthFreq(44_100)
       
        F88_200Hz:
          SynthFreq(88_200)
       
        F132_300Hz:
          SynthFreq(132_300)
          
        F48_000Hz:
          SynthFreq(48_000)
          
        F96_000Hz:
          SynthFreq(96_000)
       
        F144_000Hz:
          SynthFreq(144_000)
          
        F192_000Hz:
          SynthFreq(192_000)
       
        Other:
          SynthFreq(0)
       
      long[@ctr] := ctrReturn                        'set CTRA
      long[@frq] := frqReturn                        'set FRQA

      'fds.str(DEBUG, String("CTRA = "))        'show CTRA value
      'dio.hex(DEBUG, long[@ctr],8)     
      'fds.str(DEBUG, string("  FRQA = "))      'show FRQA value
      'dio.hex(DEBUG, long[@frq],8)
      'fds.tx(DEBUG, 13)               

      long[@cntSync] := clkfreq/1000 + cnt

      {fds.str(DEBUG, String("Cnt: "))
      dio.hex(DEBUG, cnt, 8)
      fds.str(DEBUG, String(", CntSync: "))
      dio.hex(DEBUG, long[@cntSync], 8)
      fds.str(DEBUG, String(".", 13))
      '}  
      long[@start] := 1
      waitcnt((clkfreq/100000 #> WMin) + cnt)
      long[@start] := 0
      waitcnt(long[@cntSync])
      DriverControl(Enable)
      'waitcnt(clkfreq/10 + cnt)
    
PRI DriverControl(Value)    

  CASE Value
    Enable:
      DIRA[DE1]~~
      OUTA[DE1]~~
      'fds.str(DEBUG, String("Driver DE1 pin set HIGH.", 13))

    Disable:
      DIRA[DE1]~~
      OUTA[DE1]~
      'fds.str(DEBUG, String("Driver DE1 pin set LOW.", 13))

Pri ReadRotorySwitch : value | temp, preValue
''Returns the value of the rotory position switch (0-15)
  temp := 200
  preValue := -1
  repeat until temp < 1 
    value := INA[Rot8] << 3 | INA[Rot4] << 2 | INA[Rot2] << 1 | INA[Rot1]
    value := !value & $F
    {fds.str(DEBUG, String("Value: "))
    dio.dec(DEBUG, Value)
    fds.str(DEBUG, String(".", 13)) }
    if value == preValue
      temp--
    else
      temp := 200
    preValue := value

  return    


PRI SynthFreq(Freq) | s, d

'' Determine CTR settings for synthesis of 0..128 MHz in 1 Hz steps
''
'' in:    Pin = pin to output frequency on
''        Freq = actual Hz to synthesize
''
'' out:   ctr and frq hold CTRA/CTRB and FRQA/FRQB values
''
''   Uses NCO mode %00100 for 0..499_999 Hz
''   Uses PLL mode %00010 for 500_000..128_000_000 Hz
''

  Freq := Freq #> 0 <# 128_000_000     'limit frequency range
  
  if Freq < 500_000                    'if 0 to 499_999 Hz,
    ctrReturn := constant(%00100 << 26)      '..set NCO mode
    s := 1                             '..shift = 1

  else                                 'if 500_000 to 128_000_000 Hz,
    ctrReturn := constant(%00010 << 26)      '..set PLL mode
    d := >|((Freq - 1) / 1_000_000)    'determine PLLDIV
    s := 4 - d                         'determine shift
    ctrReturn |= d << 23                     'set PLLDIV
    
  frqReturn := fraction(Freq, CLKFREQ, s)    'Compute FRQA/FRQB value
  'ctrReturn |= Pin                           'set PINA to complete CTRA/CTRB value


  
PRI fraction(a, b, shift) : f

  if shift > 0                         'if shift, pre-shift a or b left
    a <<= shift                        'to maintain significant bits while 
  if shift < 0                         'insuring proper result
    b <<= -shift
 
  repeat 32                            'perform long division of a/b
    f <<= 1
    if a => b
      a -= b
      f++           
    a <<= 1


DAT

ctr     long  $00000000
frq     long  $00000000
start   long  $00000000
cntSync long  $00000000       
        